@@ -20,6 +20,8 @@ from spyne.model.fault import Fault
 from rest_client import RestClient
 from .consts import ADDRESS
 from .consts import API_REGISTRY
+from .consts import CONFIG
+from .consts import HOSTNAME
 from .consts import HTTP_VERBS
 from .consts import REQUESTS_LOGS
 
@@ -48,12 +50,14 @@ def verify_token(source, token, **kwargs):
     l'annuaire LDAP et le résultat est stocké dans le registre d'APIs."""
 
     key = 'token:{}'.format(token)
+    env = kwargs.get('env', None)
     service = kwargs.get('service')
-    uri = kwargs.get('uri', '')
+    path = kwargs.get('path', '')
     headers = kwargs.get('headers', {'X-Real-Ip': ADDRESS})
-    ldap_api = Api('ldap', version=2)
+    ldap_api = Api('ldap', env=env)
     username = None
-    if service == 'ldap' and uri.startswith('/auth'):
+
+    if service == 'ldap' and path.startswith('/auth'):
         return True  # on laisse passer la requête telle quelle
     elif source not in API_REGISTRY:
         # la requête provient d'une source extérieure
@@ -104,35 +108,72 @@ class Api(object):
 
     """Représente une API enregistrée sur ce gateway"""
 
-    def __init__(self, service, host=None, sub_service=None, version=1):
+    def __init__(self, service, version=None, env=None):
         """Le constructeur accepte quatre variables:
               - une désignant le service
-              - une désignant l'hôte du service
-              - une désignant le sous service
-              - une désignant la version du service
+              - une désignant l'environnement du service
+              - une désignant la version du service (au format vX)
         """
         registry_key = 'service:{service}'.format(service=service)
-        infos = API_REGISTRY.hgetall(registry_key)
+        self.env = env
+        if not self.env:
+            infos = API_REGISTRY.hgetall(registry_key)
+        elif not HOSTNAME.startswith('api'):
+            # ça veut dire qu'on essaye d'appeler une API
+            # à partir d'un serveur autre que api.blueline.mg
+            # en précisant l'environnement
+            raise ValueError(
+                "Inutile de préciser {}".format(self.env)
+                )
+        else:
+            # on a précisé un environnement
+            # et on se trouve sur api.blueline.mg
+            # donc on va lire le registre de l'environnement
+            import redis
+            registry = redis.Redis(
+                host=CONFIG['API_REGISTRIES'][self.env],
+                db=15
+                )
+            infos = registry.hgetall(registry_key)
+
         if not infos:
+            # le service n'est pas déclaré dans le registre
             raise KeyError(
                 "Le service '{}' n'est pas déclaré".format(service)
                 )
         else:
             self.service = service
-            self.version = version
-            self.async = False
-            if self.service.startswith('async_'):
-                self.service = self.service.replace('async_', '')
-                self.async = True
-            self.sub_service = sub_service or ''
+
             for key, value in infos.items():
                 # chaque info contenu dans la base redis pour ce service
                 # devient un attribut
+                # les attributs qu'un service déclare sont les suivants:
+                #  - host
+                #  - service_code
+                #  - version
                 setattr(self, key.decode('utf-8'), value.decode('utf-8'))
-            if not host and hasattr(self, 'host'):
+
+            if not version:
+                if not hasattr(self, 'version'):
+                    self.version = 'v1'
+                else:
+                    self.version = 'v{}'.format(self.version)
+                    # dans le registre la version est déclarée comme un entier
+            else:
+                self.version = 'v{}'.format(self.version)
+
+            if not hasattr(self, 'host'):
+                raise ValueError(
+                    "Le service {} n'a aucun hôte déclaré".format(service)
+                    )
+            else:
                 self.host = self.host.split('.')[0]
-            elif host:
-                self.host = host
+
+            if not hasattr(self, 'service_code'):
+                raise ValueError(
+                    "Le service {} n'a aucun 'service_code'".format(service)
+                    )
+
             self.location = (
                 'https://{host}.blueline.mg:{port}'.format(
                     host=self.host,
@@ -151,14 +192,8 @@ class Api(object):
         auth = kwargs.get('auth')
         params = kwargs.get('params')
         json_doc = kwargs.get('json')
-        url = self.location
-        if self.sub_service:
-            url = '{url}/{sub_service}'.format(
-                url=url,
-                sub_service=self.subservice
-                )
-        url = '{url}/api/v{version}'.format(
-            url=url,
+        url = '{location}/api/{version}'.format(
+            location=self.location,
             version=self.version
             )
         # avant d'appeler une ressource, on doit vérifier le token
@@ -173,7 +208,8 @@ class Api(object):
                 headers['X-Real-Ip'],
                 token,
                 service=self.service,
-                uri=ressource
+                path=ressource,
+                env=self.env
                 )
         if not username:
             # le token n'a pas pu être vérifié, il faut renvoyer un HTTP 401
@@ -190,7 +226,7 @@ class Api(object):
                     },
                 'headers': headers
                 }
-            # self.record_result(method, url, result)
+            self.record_result(method, url, result)
             return result
         elif username and not isinstance(username, bool):
             auth = (username, auth[1])
@@ -203,7 +239,15 @@ class Api(object):
             params=params,
             json=json_doc
         )
-        # self.record_result(method, url, result)
+        self.record_result(method, url, result)
+        content = result['content']
+        if isinstance(content, dict) and result['status'] == 500:
+            message = content['message']
+            if isinstance(message, dict):
+                content['message'] = (
+                    'une erreur interne est survenue; '
+                    'veuillez contacter support_n1@si.blueline.mg'
+                    )
         return result
 
     def record_result(self, method, url, result):
@@ -243,9 +287,15 @@ class SoapPayload(object):
             raise ValueError('Check the tags in your XML document')
 
         self.source = source
-        self.version = self.payload['root']['header']['version']
+        self.version = 'v{}'.format(self.payload['root']['header']['version'])
         self.method = self.payload['root']['header']['param1'].lower()
         self.service = self.payload['root']['header']['param2']
+        self.env = None
+        if '/' in self.service:
+            env, service = self.service.split('/')[-2:]
+            if not env:
+                self.env = None
+            self.service = service
         self.data = self.payload['root'].get('data')
         if not self.data:
             self.data = {}
@@ -273,7 +323,7 @@ class SoapPayload(object):
         soap_body = etree.tostring(ctx.in_body_doc)
         soap_body = xmltodict.parse(soap_body)
 
-        # soap_body would look like this
+        # soap_body ressemblerait à ceci
         #   <ns0:BubbleRequest>
         #     <ns0:xml_in>
         #       <root>
@@ -281,7 +331,7 @@ class SoapPayload(object):
         #           <version>1</version>
         #           <param1>GET</param1>
         #           <param2>some_service</param2>
-        #           <param3>some/resource<param3>
+        #           <param3>/some/resource<param3>
         #           <ident>username_or_token</ident>
         #           <psw>password_if_necessary</psw>
         #         </header>
@@ -348,7 +398,7 @@ class SoapPayload(object):
         En fait on rappelle sponge, mais sur son interface
         REST cette fois.
         """
-        api = Api(self.service, version=self.version)
+        api = Api(self.service, version=self.version, env=self.env)
 
         # récupération de l'adresse complète de la ressource
         resource = '{param3}/{params}'.format(
@@ -356,6 +406,8 @@ class SoapPayload(object):
             params='/'.join(self.data.get('resource_params', {}).values())
         )
         resource = resource.rstrip('/')
+        if not resource.startswith('/'):
+            resource = '/' + resource
 
         # récupération des credentials
         password = self.payload['root']['header']['psw']
